package com.jiuqi.util;

import java.text.DecimalFormat;
import java.util.Stack;
import java.text.NumberFormat;
import java.util.HashMap;

public class NumberFormatManager
{
    private static HashMap integerFormats;
    private static HashMap numberFormats;
    private static HashMap patternFormats;
    private static final Integer[] INT_VALUES;
    
    static {
        NumberFormatManager.integerFormats = new HashMap();
        NumberFormatManager.numberFormats = new HashMap();
        NumberFormatManager.patternFormats = new HashMap();
        INT_VALUES = new Integer[] { new Integer(0), new Integer(1), new Integer(2), new Integer(3), new Integer(4), new Integer(5), new Integer(6), new Integer(7), new Integer(8), new Integer(9), new Integer(10), new Integer(11), new Integer(12), new Integer(13), new Integer(14), new Integer(15), new Integer(16), new Integer(17), new Integer(18), new Integer(19), new Integer(20), new Integer(21), new Integer(22), new Integer(23), new Integer(24), new Integer(25), new Integer(26), new Integer(27), new Integer(28), new Integer(29), new Integer(30), new Integer(31), new Integer(32), new Integer(33), new Integer(34), new Integer(35), new Integer(36), new Integer(37), new Integer(38), new Integer(39) };
    }
    
    public static NumberFormat createIntegerFormat(final int integerDigits) {
        NumberFormat format = null;
        final Integer key = getInt(integerDigits);
        synchronized (NumberFormatManager.integerFormats) {
            Stack formats = NumberFormatManager.integerFormats.get(key);
            if (formats == null) {
                formats = new Stack();
                NumberFormatManager.integerFormats.put(key, formats);
            }
            if (!formats.isEmpty()) {
                format = formats.pop();
            }
            else {
                format = NumberFormat.getInstance();
                format.setMinimumFractionDigits(0);
                format.setMaximumFractionDigits(0);
                format.setMinimumIntegerDigits(integerDigits);
                format.setGroupingUsed(false);
            }
            monitorexit(NumberFormatManager.integerFormats);
        }
        return format;
    }
    
    public static void recycleIntegerFormat(final int integerDigits, final NumberFormat format) {
        final Integer key = getInt(integerDigits);
        synchronized (NumberFormatManager.integerFormats) {
            final Stack formats = NumberFormatManager.integerFormats.get(key);
            if (formats != null) {
                formats.push(format);
            }
            monitorexit(NumberFormatManager.integerFormats);
        }
    }
    
    public static NumberFormat createNumberFormat(final int fractionDigits, final boolean groupingUsed) {
        return createNumberFormat(fractionDigits, fractionDigits, groupingUsed);
    }
    
    public static NumberFormat createNumberFormat(final int minimumFractionDigits, final int maximumFractionDigits, final boolean groupingUsed) {
        int keyData = groupingUsed ? 1 : 0;
        keyData |= (maximumFractionDigits << 16 | minimumFractionDigits << 8);
        final Integer key = new Integer(keyData);
        NumberFormat format = null;
        synchronized (NumberFormatManager.numberFormats) {
            Stack formats = NumberFormatManager.numberFormats.get(key);
            if (formats == null) {
                formats = new Stack();
                NumberFormatManager.numberFormats.put(key, formats);
            }
            if (!formats.isEmpty()) {
                format = formats.pop();
            }
            else {
                format = NumberFormat.getInstance();
                format.setMaximumFractionDigits(maximumFractionDigits);
                format.setMinimumFractionDigits(minimumFractionDigits);
                format.setMinimumIntegerDigits(1);
                format.setGroupingUsed(groupingUsed);
            }
            monitorexit(NumberFormatManager.numberFormats);
        }
        return format;
    }
    
    public static void recycleNumberFormat(final int fractionDigits, final boolean groupingUsed, final NumberFormat format) {
        recycleNumberFormat(fractionDigits, fractionDigits, groupingUsed, format);
    }
    
    public static void recycleNumberFormat(final int minimumFractionDigits, final int maximumFractionDigits, final boolean groupingUsed, final NumberFormat format) {
        int keyData = groupingUsed ? 1 : 0;
        keyData |= (maximumFractionDigits << 16 | minimumFractionDigits << 8);
        final Integer key = new Integer(keyData);
        synchronized (NumberFormatManager.numberFormats) {
            final Stack formats = NumberFormatManager.numberFormats.get(key);
            if (formats != null) {
                formats.push(format);
            }
            monitorexit(NumberFormatManager.numberFormats);
        }
    }
    
    public static NumberFormat createNumberFormat(final String pattern) {
        NumberFormat format = null;
        synchronized (NumberFormatManager.patternFormats) {
            Stack formats = NumberFormatManager.patternFormats.get(pattern);
            if (formats == null) {
                formats = new Stack();
                NumberFormatManager.patternFormats.put(pattern, formats);
            }
            if (!formats.isEmpty()) {
                format = formats.pop();
            }
            else {
                format = new DecimalFormat(pattern);
            }
            monitorexit(NumberFormatManager.patternFormats);
        }
        return format;
    }
    
    public static void recycleNumberFormat(final String pattern, final NumberFormat format) {
        synchronized (NumberFormatManager.patternFormats) {
            final Stack formats = NumberFormatManager.patternFormats.get(pattern);
            if (formats != null) {
                formats.push(format);
            }
            monitorexit(NumberFormatManager.patternFormats);
        }
    }
    
    public static final Integer getInt(final int index) {
        return (index < NumberFormatManager.INT_VALUES.length) ? NumberFormatManager.INT_VALUES[index] : new Integer(index);
    }
    
    public static void main(final String[] args) {
        for (int i = 0; i < 2000; ++i) {
            final NumberFormatManagerTest1 testor = new NumberFormatManagerTest1();
            testor.start();
        }
    }
}
